@@ -32,8 +32,12 @@
 #include <glib-object.h>
 #include <glib/gi18n.h>
 
+#include <act/act-user-manager.h>
+
 struct _GdmSessionSettingsPrivate
 {
+        ActUserManager *user_manager;
+        ActUser *user;
         char *session_name;
         char *language_name;
 };
@@ -55,6 +59,7 @@ enum {
         PROP_0 = 0,
         PROP_SESSION_NAME,
         PROP_LANGUAGE_NAME,
+        PROP_IS_LOADED
 };
 
 G_DEFINE_TYPE (GdmSessionSettings, gdm_session_settings, G_TYPE_OBJECT)
@@ -93,6 +98,10 @@ gdm_session_settings_class_install_properties (GdmSessionSettingsClass *settings
                                         NULL,
                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
         g_object_class_install_property (object_class, PROP_LANGUAGE_NAME, param_spec);
+
+        param_spec = g_param_spec_boolean ("is-loaded", NULL, NULL,
+                                           FALSE, G_PARAM_READABLE);
+        g_object_class_install_property (object_class, PROP_IS_LOADED, param_spec);
 }
 
 static void
@@ -102,6 +111,8 @@ gdm_session_settings_init (GdmSessionSettings *settings)
                                                      GDM_TYPE_SESSION_SETTINGS,
                                                      GdmSessionSettingsPrivate);
 
+        settings->priv->user_manager = act_user_manager_get_default ();
+
 }
 
 static void
@@ -112,6 +123,10 @@ gdm_session_settings_finalize (GObject *object)
 
         settings = GDM_SESSION_SETTINGS (object);
 
+        if (settings->priv->user != NULL) {
+                g_object_unref (settings->priv->user);
+        }
+
         g_free (settings->priv->session_name);
         g_free (settings->priv->language_name);
 
@@ -205,6 +220,10 @@ gdm_session_settings_get_property (GObject    *object,
                         g_value_set_string (value, settings->priv->language_name);
                 break;
 
+                case PROP_IS_LOADED:
+                        g_value_set_boolean (value, gdm_session_settings_is_loaded (settings));
+                break;
+
                 default:
                         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -224,127 +243,116 @@ gdm_session_settings_new (void)
 gboolean
 gdm_session_settings_is_loaded (GdmSessionSettings  *settings)
 {
-        return settings->priv->session_name != NULL ||
-               settings->priv->language_name != NULL;
+        if (settings->priv->user == NULL) {
+                return FALSE;
+        }
+
+        return act_user_is_loaded (settings->priv->user);
+}
+
+static void
+load_settings_from_user (GdmSessionSettings *settings)
+{
+        const char *session_name;
+        const char *language_name;
+
+        if (!act_user_is_loaded (settings->priv->user)) {
+                g_warning ("GdmSessionSettings: trying to load user settings from unloaded user");
+                return;
+        }
+
+        session_name = act_user_get_x_session (settings->priv->user);
+
+        if (session_name != NULL) {
+                gdm_session_settings_set_session_name (settings, session_name);
+        }
+
+        language_name = act_user_get_language (settings->priv->user);
+
+        if (language_name != NULL) {
+                gdm_session_settings_set_language_name (settings, language_name);
+        }
+
+        g_object_notify (G_OBJECT (settings), "is-loaded");
+}
+
+static void
+on_user_is_loaded_changed (ActUser            *user,
+                           GParamSpec         *pspec,
+                           GdmSessionSettings *settings)
+{
+        if (act_user_is_loaded (settings->priv->user)) {
+                load_settings_from_user (settings);
+        }
 }
 
 gboolean
 gdm_session_settings_load (GdmSessionSettings  *settings,
-                           const char          *home_directory,
-                           GError             **error)
+                           const char          *username)
 {
-        GKeyFile *key_file;
-        GError   *load_error;
-        gboolean  is_loaded;
-        char     *session_name;
-        char     *language_name;
-        char     *filename;
+        ActUser *old_user;
 
         g_return_val_if_fail (settings != NULL, FALSE);
-        g_return_val_if_fail (home_directory != NULL, FALSE);
+        g_return_val_if_fail (username != NULL, FALSE);
         g_return_val_if_fail (!gdm_session_settings_is_loaded (settings), FALSE);
-        filename = g_build_filename (home_directory, ".dmrc", NULL);
 
-        is_loaded = FALSE;
-        key_file = g_key_file_new ();
+        if (settings->priv->user != NULL) {
+                old_user = settings->priv->user;
 
-        load_error = NULL;
-        if (!g_key_file_load_from_file (key_file, filename,
-                                        G_KEY_FILE_NONE, &load_error)) {
-                g_propagate_error (error, load_error);
-                goto out;
-        }
-
-        session_name = g_key_file_get_string (key_file, "Desktop", "Session",
-                                              &load_error);
-
-        if (session_name != NULL) {
-                gdm_session_settings_set_session_name (settings, session_name);
-                g_free (session_name);
-        } else if (g_error_matches (load_error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-                g_error_free (load_error);
-                load_error = NULL;
+                g_signal_handlers_disconnect_by_func (G_OBJECT (settings->priv->user),
+                                                      G_CALLBACK (on_user_is_loaded_changed),
+                                                      settings);
         } else {
-                g_propagate_error (error, load_error);
-                goto out;
+                old_user = NULL;
         }
 
-        language_name = g_key_file_get_string (key_file, "Desktop", "Language",
-                                               &load_error);
-
-        if (language_name != NULL) {
-                gdm_session_settings_set_language_name (settings, language_name);
-                g_free (language_name);
-        } else if (g_error_matches (load_error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-                g_error_free (load_error);
-                load_error = NULL;
-        } else {
-                g_propagate_error (error, load_error);
-                goto out;
+        settings->priv->user = act_user_manager_get_user (settings->priv->user_manager,
+                                                          username);
+        if (old_user != NULL) {
+                g_object_unref (old_user);
+                old_user = NULL;
         }
 
-        is_loaded = TRUE;
-out:
-        g_key_file_free (key_file);
-        g_free (filename);
+        if (!act_user_is_loaded (settings->priv->user)) {
+                g_signal_connect (settings->priv->user,
+                                  "notify::is-loaded",
+                                  G_CALLBACK (on_user_is_loaded_changed),
+                                  settings);
+                return FALSE;
+        }
 
-        return is_loaded;
+        load_settings_from_user (settings);
+
+        return TRUE;
 }
 
 gboolean
 gdm_session_settings_save (GdmSessionSettings  *settings,
-                           const char          *home_directory,
-                           GError             **error)
+                           const char          *username)
 {
-        GKeyFile *key_file;
-        GError   *file_error;
-        gboolean  is_saved;
-        char     *filename;
-        gsize     length;
-        gchar    *contents;
+        ActUser  *user;
 
         g_return_val_if_fail (GDM_IS_SESSION_SETTINGS (settings), FALSE);
-        g_return_val_if_fail (home_directory != NULL, FALSE);
+        g_return_val_if_fail (username != NULL, FALSE);
         g_return_val_if_fail (gdm_session_settings_is_loaded (settings), FALSE);
-        filename = g_build_filename (home_directory, ".dmrc", NULL);
 
-        is_saved = FALSE;
-        key_file = g_key_file_new ();
+        user = act_user_manager_get_user (settings->priv->user_manager,
+                                          username);
 
-        file_error = NULL;
-        g_key_file_load_from_file (key_file, filename,
-                                   G_KEY_FILE_KEEP_COMMENTS |
-                                   G_KEY_FILE_KEEP_TRANSLATIONS,
-                                   NULL);
+
+        if (!act_user_is_loaded (user)) {
+                g_object_unref (user);
+                return FALSE;
+        }
 
         if (settings->priv->session_name != NULL) {
-                g_key_file_set_string (key_file, "Desktop", "Session",
-                                       settings->priv->session_name);
+                act_user_set_x_session (user, settings->priv->session_name);
         }
 
         if (settings->priv->language_name != NULL) {
-                g_key_file_set_string (key_file, "Desktop", "Language",
-                                       settings->priv->language_name);
+                act_user_set_language (user, settings->priv->language_name);
         }
+        g_object_unref (user);
 
-        contents = g_key_file_to_data (key_file, &length, &file_error);
-
-        if (contents == NULL) {
-                g_propagate_error (error, file_error);
-                goto out;
-        }
-
-        if (!g_file_set_contents (filename, contents, length, &file_error)) {
-                g_free (contents);
-                g_propagate_error (error, file_error);
-                goto out;
-        }
-        g_free (contents);
-
-        is_saved = TRUE;
-out:
-        g_key_file_free (key_file);
-        g_free (filename);
-
-        return is_saved;
+        return TRUE;
 }
