@@ -56,6 +56,7 @@
 
 #include "gdm-greeter-login-window.h"
 #include "gdm-user-chooser-widget.h"
+#include "gdm-session-option-widget.h"
 
 #ifdef HAVE_PAM
 #include <security/pam_appl.h>
@@ -85,6 +86,7 @@
 #define LSB_RELEASE_COMMAND "lsb_release -d"
 
 #define GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_LOGIN_WINDOW, GdmGreeterLoginWindowPrivate))
+#define GDM_CUSTOM_SESSION "custom"
 
 enum {
         MODE_UNDEFINED = 0,
@@ -102,6 +104,7 @@ enum {
 struct GdmGreeterLoginWindowPrivate
 {
         GtkBuilder      *builder;
+        GtkWidget       *session_option_widget;
         GtkWidget       *user_chooser;
         GtkWidget       *auth_banner_label;
         GtkWidget       *current_button;
@@ -143,7 +146,7 @@ enum {
         QUERY_ANSWER,
         START_SESSION,
         USER_SELECTED,
-        DISCONNECTED,
+        SESSION_SELECTED,
         CANCELLED,
         LAST_SIGNAL
 };
@@ -506,12 +509,15 @@ switch_mode (GdmGreeterLoginWindow *login_window,
         switch (number) {
         case MODE_SELECTION:
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
+                gtk_widget_hide (login_window->priv->session_option_widget);
                 break;
         case MODE_TIMED_LOGIN:
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_TIMED_LOGIN);
+                gtk_widget_show (login_window->priv->session_option_widget);
                 break;
         case MODE_AUTHENTICATION:
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
+                gtk_widget_show (login_window->priv->session_option_widget);
                 break;
         default:
                 g_assert_not_reached ();
@@ -624,7 +630,6 @@ reset_dialog (GdmGreeterLoginWindow *login_window,
 {
         GtkWidget  *entry;
         GtkWidget  *label;
-        guint       mode;
 
         g_debug ("GdmGreeterLoginWindow: Resetting dialog to mode %u", dialog_mode);
         set_busy (login_window);
@@ -705,6 +710,7 @@ gdm_greeter_login_window_ready (GdmGreeterLoginWindow *login_window)
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
+        gdk_window_beep (gtk_widget_get_window (GTK_WIDGET (login_window)));
 
         /* If we are retrying a previously selected user */
         if (!login_window->priv->user_list_disabled &&
@@ -879,6 +885,7 @@ gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
         gtk_label_set_text (GTK_LABEL (label), text);
 
         show_widget (login_window, "auth-input-box", TRUE);
+
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
@@ -909,6 +916,7 @@ gdm_greeter_login_window_secret_info_query (GdmGreeterLoginWindow *login_window,
         gtk_label_set_text (GTK_LABEL (label), text);
 
         show_widget (login_window, "auth-input-box", TRUE);
+        gtk_widget_show (login_window->priv->session_option_widget);
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
@@ -1030,7 +1038,6 @@ on_user_chosen (GdmUserChooserWidget  *user_chooser,
                 GdmGreeterLoginWindow *login_window)
 {
         char *user_name;
-        guint mode;
 
         user_name = gdm_user_chooser_widget_get_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser));
         g_debug ("GdmGreeterLoginWindow: user chosen '%s'", user_name);
@@ -1050,6 +1057,46 @@ on_user_unchosen (GdmUserChooserWidget  *user_chooser,
         do_cancel (login_window);
 }
 
+static void
+on_session_activated (GdmSessionOptionWidget *session_option_widget,
+                      GdmGreeterLoginWindow  *login_window)
+{
+        char *session;
+
+        session = gdm_session_option_widget_get_current_session (GDM_SESSION_OPTION_WIDGET (login_window->priv->session_option_widget));
+        if (session == NULL) {
+                return;
+        }
+
+        g_signal_emit (login_window, signals[SESSION_SELECTED], 0, session);
+
+        g_free (session);
+}
+
+void
+gdm_greeter_login_window_set_default_session_name (GdmGreeterLoginWindow *login_window,
+                                                   const char            *session_name)
+{
+        g_return_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window));
+
+        if (session_name != NULL && !gdm_option_widget_lookup_item (GDM_OPTION_WIDGET (login_window->priv->session_option_widget),
+                                            session_name, NULL, NULL, NULL)) {
+                if (strcmp (session_name, GDM_CUSTOM_SESSION) == 0) {
+                        gdm_option_widget_add_item (GDM_OPTION_WIDGET (login_window->priv->session_option_widget),
+                                                    GDM_CUSTOM_SESSION,
+                                                    C_("customsession", "Custom"),
+                                                    _("Custom session"),
+                                                    GDM_OPTION_WIDGET_POSITION_TOP);
+                } else {
+                        g_warning ("Default session is not available");
+                        return;
+                }
+        }
+
+        gdm_option_widget_set_default_item (GDM_OPTION_WIDGET (login_window->priv->session_option_widget),
+                                            session_name);
+}
+
 static void
 rotate_computer_info (GdmGreeterLoginWindow *login_window)
 {
@@ -1266,7 +1313,7 @@ load_theme (GdmGreeterLoginWindow *login_window)
         box = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "window-frame"));
         gtk_container_add (GTK_CONTAINER (login_window), box);
 
-        /* FIXME: user chooser should implement GtkBuildable and this should get dropped
+        /* FIXME: user and session chooser should get loaded from ui file instead
          */
         login_window->priv->user_chooser = gdm_user_chooser_widget_new ();
         box = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "selection-box"));
@@ -1275,6 +1322,7 @@ load_theme (GdmGreeterLoginWindow *login_window)
 
         gdm_user_chooser_widget_set_show_only_chosen (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
 
+
         g_signal_connect (login_window->priv->user_chooser,
                           "loaded",
                           G_CALLBACK (on_users_loaded),
@@ -1293,6 +1341,22 @@ load_theme (GdmGreeterLoginWindow *login_window)
                                  G_CALLBACK (on_user_chooser_visibility_changed),
                                  login_window);
 
+        login_window->priv->session_option_widget = gdm_session_option_widget_new ();
+        box = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "buttonbox"));
+        g_object_set (G_OBJECT (login_window->priv->session_option_widget),
+                      "xscale", 0.0,
+                      "yscale", 0.0,
+                      "xalign", 0.0,
+                      "yalign", 1.0,
+                      NULL);
+        gtk_container_add (GTK_CONTAINER (box), login_window->priv->session_option_widget);
+        gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (box), login_window->priv->session_option_widget, TRUE);
+
+        g_signal_connect (login_window->priv->session_option_widget,
+                          "activated",
+                          G_CALLBACK (on_session_activated),
+                          login_window);
+
         login_window->priv->auth_banner_label = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "auth-banner-label"));
         /*make_label_small_italic (login_window->priv->auth_banner_label);*/
 
@@ -1329,7 +1393,7 @@ gdm_greeter_login_window_key_press_event (GtkWidget   *widget,
 
         login_window = GDM_GREETER_LOGIN_WINDOW (widget);
 
-        if (event->keyval == GDK_Escape) {
+        if (event->keyval == GDK_KEY_Escape) {
                 if (login_window->priv->dialog_mode == MODE_AUTHENTICATION
                     || login_window->priv->dialog_mode == MODE_TIMED_LOGIN) {
                         do_cancel (GDM_GREETER_LOGIN_WINDOW (widget));
@@ -1340,45 +1404,82 @@ gdm_greeter_login_window_key_press_event (GtkWidget   *widget,
 }
 
 static void
-gdm_greeter_login_window_size_request (GtkWidget      *widget,
-                                       GtkRequisition *requisition)
+gdm_greeter_login_window_get_preferred_width (GtkWidget *widget,
+                                              gint      *minimum_size,
+                                              gint      *natural_size)
 {
         int             monitor;
         GdkScreen      *screen;
-        GtkRequisition  child_requisition;
         GdkRectangle    area;
+        GtkAllocation   widget_allocation;
+        int             min_size;
+        int             nat_size;
 
-        if (GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->size_request) {
-                GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->size_request (widget, requisition);
-        }
+        min_size = 0;
+        nat_size = 0;
 
         if (!gtk_widget_get_realized (widget)) {
-                return;
+                goto out;
         }
 
+        gtk_widget_get_preferred_width (gtk_bin_get_child (GTK_BIN (widget)),
+                                        &min_size,
+                                        &nat_size);
+
+        /* Make width be at least 33% screen width */
         screen = gtk_widget_get_screen (widget);
         monitor = gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (widget));
         gdk_screen_get_monitor_geometry (screen, monitor, &area);
+        min_size = MAX (min_size, .33 * area.width);
+        nat_size = MAX (nat_size, .33 * area.width);
 
-        gtk_widget_get_child_requisition (gtk_bin_get_child (GTK_BIN (widget)), &child_requisition);
-        *requisition = child_requisition;
-
-        guint border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-        requisition->width += 2 * border_width;
-        requisition->height += 2 * border_width;
-
-        /* Make width be at least 33% screen width
-         * and height be at most 80% of screen height
-         */
-        requisition->width = MAX (requisition->width, .33 * area.width);
-        requisition->height = MIN (requisition->height, .80 * area.height);
-
-       /* Don't ever shrink window width
-        */
-        GtkAllocation widget_allocation;
+       /* Don't ever shrink window width */
         gtk_widget_get_allocation (widget, &widget_allocation);
 
-        requisition->width = MAX (requisition->width, widget_allocation.width);
+        min_size = MAX (min_size, widget_allocation.width);
+        nat_size = MAX (nat_size, widget_allocation.width);
+
+ out:
+        if (minimum_size)
+                *minimum_size = min_size;
+        if (natural_size)
+                *natural_size = nat_size;
+}
+
+static void
+gdm_greeter_login_window_get_preferred_height (GtkWidget *widget,
+                                               gint      *minimum_size,
+                                               gint      *natural_size)
+{
+        int             monitor;
+        GdkScreen      *screen;
+        GdkRectangle    area;
+        int             min_size;
+        int             nat_size;
+
+        min_size = 0;
+        nat_size = 0;
+
+        if (!gtk_widget_get_realized (widget)) {
+                goto out;
+        }
+
+        gtk_widget_get_preferred_height (gtk_bin_get_child (GTK_BIN (widget)),
+                                        &min_size,
+                                        &nat_size);
+
+        /* Make height be at most 80% of screen height */
+        screen = gtk_widget_get_screen (widget);
+        monitor = gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (widget));
+        gdk_screen_get_monitor_geometry (screen, monitor, &area);
+        min_size = MIN (min_size, .8 * area.height);
+        nat_size = MIN (nat_size, .8 * area.height);
+
+ out:
+        if (minimum_size)
+                *minimum_size = min_size;
+        if (natural_size)
+                *natural_size = nat_size;
 }
 
 static void
@@ -1463,6 +1564,7 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
         GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+        GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
 
         object_class->get_property = gdm_greeter_login_window_get_property;
         object_class->set_property = gdm_greeter_login_window_set_property;
@@ -1470,7 +1572,10 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
         object_class->finalize = gdm_greeter_login_window_finalize;
 
         widget_class->key_press_event = gdm_greeter_login_window_key_press_event;
-        widget_class->size_request = gdm_greeter_login_window_size_request;
+        widget_class->get_preferred_width = gdm_greeter_login_window_get_preferred_width;
+        widget_class->get_preferred_height = gdm_greeter_login_window_get_preferred_height;
+
+        gtk_container_class_handle_border_width (container_class);
 
         signals [BEGIN_AUTO_LOGIN] =
                 g_signal_new ("begin-auto-login",
@@ -1521,6 +1626,16 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
+        signals [SESSION_SELECTED] =
+                g_signal_new ("session-selected",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, session_selected),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1, G_TYPE_STRING);
         signals [CANCELLED] =
                 g_signal_new ("cancelled",
                               G_TYPE_FROM_CLASS (object_class),
@@ -1531,16 +1646,6 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
-        signals [DISCONNECTED] =
-                g_signal_new ("disconnected",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, disconnected),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
         signals [START_SESSION] =
                 g_signal_new ("start-session",
                               G_TYPE_FROM_CLASS (object_class),
